﻿/*
 * To do : 
 * 
 * Dateiüberwachung für Konfigdateien
 * X-Achse Beschriftung scheisse bei Zoom, gleiches bei Grundlast
 * Probleme im webserver wg. Datenübermittlung
 * Statistikseite vervollständigen (Abrechnung, Zählerstand, Kontostand)
 * Statistikknopf (Min-Max, Schnitt für die angegebenen Kurven)
 * Exportknopf (Daten in CSV exportieren)
 * 
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Net;
using ZedGraph;
 
// namespace WindowsFormsApplication1
namespace Energiemonitor
{

    //---------------
    // Hauptformular
    //---------------
    public partial class Form1 : Form
    {

        //  Liste der eingeladenen Rohdaten (Klasse Zählerdaten)
        List<Zaehlerdaten> lRohdaten = new List<Zaehlerdaten>();
        
        // Liste der Kostendaten
        List<Kosten> lKostendaten = new List<Kosten>();

        // Liste der Ablesedaten
        List<Ablesungen> lAblesedaten = new List<Ablesungen>();
       
        // Liste mit den Abschlagsdaten
        List<Abschlaege> lAbschlaege = new List<Abschlaege>();

        // Lokal berechnete Verbräuche
        float Strom = 0;
        float Gas = 0;
        float Wasser = 0;

        // Konfigurationsdaten
        string WebServerIP = "0.0.0.1";
        uint GrundLastStart = 22 * 60 * 60;        // 23 Uhr Grundlastbeginn
        uint GrundLastEnde = 5 * 60 * 60;          // 5 Uhr Grundlastbeginn

        // Grafikdaten
        public uint GrafikStartZeitraum = 0;               // Wo geht die Grafik los ?
        public uint GrafikEndeZeitraum = 0;                // Wo hört sie wieder auf ?
        public bool GrafikStrom = true;                   // Stromkurve anzeigen ja/nein
        public bool GrafikGas = false;                     // Gaskurve anzeigen 
        public bool GrafikWasser = false;                  // Wasserkurve anzeigen 
        public int GrafikGrundlast = 0;               // Grundlasten mit anzeigen ja/nein
  
        // Temperaturdaten
        PointPairList KurveAussen;
        PointPairList KurveHeizungVorlauf;
        PointPairList KurveHeizungRuecklauf;
        PointPairList KurveBoilerEinlass;
        PointPairList KurveBoilerAuslass;
        PointPairList KurveAbgasHeizung;
        PointPairList KurveAbgasBoiler;
        PointPairList KurveHeizungsraum;
        
        YAxis yAxis3 = new YAxis("Temperatur [°C]");

        GraphPane Grafik;

        // Datenlisten für die grafische Kurvenanzeige
        PointPairList Stromkurvendaten;
        PointPairList Gaskurvendaten; 
        PointPairList Wasserkurvendaten;
        PointPairList Grundlastkurvendaten;
        PointPairList GrundlastKurvenDatenGlatt;

        // Checkbox für Temperaturauswahl
        int[] TCheckbox;
        int[] TCheckboxAlt;

        double StromMax, GasMax, WasserMax, GrundlastMax;
        uint GrafikXStart, GrafikXEnde;

        //------------------------------
        // Lädt die Konfigurationsdatei
        //------------------------------
        void LadeKonfiguration ()
        {
            string templine;
            string[] arr;

            System.IO.StreamReader file = new System.IO.StreamReader("config.txt");
            while (!file.EndOfStream) 
            { 
                templine = file.ReadLine();
                if ((templine != "") && (templine.IndexOf ("#") < 0))
                {
                    // Webserver-IP
                    if (templine == "[IP]") WebServerIP = file.ReadLine();

                    // Bestrimmung der Grundlastzeiten
                    if (templine == "[Grundlast]")
                    {
                        templine = file.ReadLine();
                        arr = templine.Split(' ');
                        string[] TimeStart = arr[0].Split(':');
                        GrundLastStart = (uint)Convert.ToInt32(TimeStart[0]) * 60 * 60 + (uint)Convert.ToInt32(TimeStart[1]) * 60;
                        string[] TimeEnde = arr[1].Split(':');
                        GrundLastEnde = (uint)Convert.ToInt32(TimeEnde[0]) * 60 * 60 + (uint)Convert.ToInt32(TimeEnde[1]) * 60;
                    }
                }
            }
            file.Close();           
 
            // Konfig-Daten anzeigen
            LabelIP.Text = WebServerIP;

            LabelGrundlastStart.Text = Convert.ToString(GrundLastStart / 3600) + ":" +
                                       ((GrundLastStart % 3600) / 60).ToString("D2") +
                                        " Uhr bis " + 
                                        Convert.ToString(GrundLastEnde / 3600) + ":" +
                                        ((GrundLastEnde % 3600) / 60).ToString("D2") +
                                        " Uhr"; 

        }

        //------------------------------------
        // Lädt die Liste mit den Kostendaten
        //------------------------------------
        void LadeKostenDaten ()
        {
            string templine;
            
            System.IO.StreamReader file = new System.IO.StreamReader("kosten.txt");
            while (!file.EndOfStream) 
            { 
                templine = file.ReadLine();
                if ((templine != "") && (templine.IndexOf ("#") < 0))
                {
                    lKostendaten.Add(new Kosten(templine));    
                }
            }
            file.Close();
            // Defekte Datensätze killen
            lKostendaten.RemoveAll(delegate(Kosten s) { return s.DatumUTC == 0; }); // 0xFFFFFFFF-Datensätze aus der Liste entfernen

            // Eingeladene Daten im letzten Stand ausgeben
            LabelstromGrundpreis.Text = lKostendaten.Last().StromGrundpreis.ToString("#,##0.00") + " € / a";
            LabelStromArbeitspreis.Text = lKostendaten.Last().StromArbeitspreis.ToString("#,##0.00") + " € / kWh";

            LabelGasGrundpreis.Text = lKostendaten.Last().GasGrundpreis.ToString("#,##0.00") + " € / a";
            LabelGasArbeitspreis.Text = lKostendaten.Last().Gararbeitspreis.ToString("#,##0.00") + " € / kWh";

            LabelWassergrundpreis.Text = lKostendaten.Last().WasserGrundpreis.ToString("#,##0.00") + " € / a";
            LabelWasserArbeitspreis.Text = lKostendaten.Last().WasserArbeitspreis.ToString("#,##0.00") + " € / qm";

        }

        //------------------------------------
        // Lädt die Liste mit den Abschlägen
        //------------------------------------
        void LadeAbschlagsDaten()
        {
            string templine;

            System.IO.StreamReader file = new System.IO.StreamReader("abschlag.txt");
            while (!file.EndOfStream)
            {
                templine = file.ReadLine();
                if ((templine != "") && (templine.IndexOf("#") < 0))
                {
                    lAbschlaege.Add(new Abschlaege(templine));
                }
            }
            file.Close();
            // Defekte Datensätze killen
            lAbschlaege.RemoveAll(delegate(Abschlaege s) { return s.Datum == 0; }); // 0xFFFFFFFF-Datensätze aus der Liste entfernen

            // Eingeladene Daten im letzten Stand ausgeben
            LabelAbschlagStrom.Text = lAbschlaege.Last().Strom.ToString("#,##0.00") + " €";
            LabelAbschlagGas.Text = lAbschlaege.Last().Gas.ToString("#,##0.00") + " €";
            LabelAbschlagWasser.Text = lAbschlaege.Last().Wasser.ToString("#,##0.00") + " €";
        }

        //------------------------------------
        // Lädt die Liste mit den  ten
        //------------------------------------
        void LadeAbleseDaten()
        {
            string templine;
            System.DateTime dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);

            System.IO.StreamReader file = new System.IO.StreamReader("ablesungen.txt");
            while (!file.EndOfStream)
            {
                templine = file.ReadLine();
                if ((templine != "") && (templine.IndexOf("#") < 0) && (templine != " ") && templine.IndexOf(" ") > 3)
                {
                    lAblesedaten.Add(new Ablesungen(templine));
                }
            }
            file.Close();
            // Defekte Datensätze killen
            lAblesedaten.RemoveAll(delegate(Ablesungen s) { return s.DatumStrom == 0; }); // 0xFFFFFFFF-Datensätze aus der Liste entfernen

            // Eingeladene Daten im letzten Stand ausgeben auf der Konfigurationseite
            LabelAblesungenStrom.Text = lAblesedaten.Last().StandStrom.ToString()+ " kWh";
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumStrom);
            LabelAblesungenStromDatum.Text = dateTime.ToShortDateString();
           
            LabelAblesungenGas.Text = lAblesedaten.Last().StandGas.ToString()+ " qm";
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumGas);
            LabelAblesungenGasDatum.Text = dateTime.ToShortDateString();
            
            LabelAblesungenWasser.Text = lAblesedaten.Last().StandWasser.ToString() + " qm";
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumWasser);
            LabelAblesungenWasserDatum.Text = dateTime.ToShortDateString();

            // Ausgaben auf der Statistikseite
            LabelAblStrom.Text = lAblesedaten.Last().StandStrom.ToString()+ " kWh";
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumStrom);
            LabelAblStromVom.Text = dateTime.ToShortDateString();

            LabelAblGas.Text = lAblesedaten.Last().StandGas.ToString()+ " qm";
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumGas);
            LabelAblGasVom.Text = dateTime.ToShortDateString();
                
            LabelAlbWasser.Text = lAblesedaten.Last().StandWasser.ToString() + " qm";
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumWasser);
            LabelAlbWasserVom.Text = dateTime.ToShortDateString();
      
        }


        //-------------------------------------------------------
        // Gespeicherte Zählerdaten einladen und Anzahl ausgeben
        //-------------------------------------------------------
        void DateiLaden()
        {
            // AnzahlRohdaten = 0;
            System.IO.StreamReader file = new System.IO.StreamReader("zaehlerdaten.txt");
            while (!file.EndOfStream) 
            { 
                lRohdaten.Add (new Zaehlerdaten(file.ReadLine())); 
            }
            file.Close();  
        }

        //---------------------------------
        // Schrottige Datensätze verwerfen
        //---------------------------------
        void Junkfilter()
        {
         lRohdaten.RemoveAll(delegate (Zaehlerdaten s)  { return s.UTC == 0xFFFFFFFF; } ); // 0xFFFFFFFF-Datensätze aus der Liste entfernen
         lRohdaten.RemoveAll(delegate(Zaehlerdaten s) { return s.Strom == 0; }); // 0xFFFFFFFF-Datensätze aus der Liste entfernen
         // Ist das letzte Element leer (Leerzeile) ?
         if (lRohdaten.Last().Datum == null) lRohdaten.RemoveAt(lRohdaten.Count - 1);
        }

        //---------------------------------------------------
        // Transiente Verbrauchsdaten in der Liste erzeugen
        //---------------------------------------------------
        void MomentDatenErrechnen()
        {
            // Startwerte ermitteln
            uint VorwertStrom = lRohdaten.First().Strom;
            uint VorwertGas = lRohdaten.First().Gas;
            uint VorwertWasser = lRohdaten.First().Wasser;

            int MaxWert = lRohdaten.Count;

            for (int i = 0; i < lRohdaten.Count-1; i++)
            {
                if (i == 12)
                {
                }

                if (lRohdaten.ElementAt(i + 1).Strom >= lRohdaten.ElementAt(i).Strom)
                 lRohdaten.ElementAt(i).MomentStrom = lRohdaten.ElementAt(i + 1).Strom - lRohdaten.ElementAt(i).Strom; 
                if (lRohdaten.ElementAt(i + 1).Gas >= lRohdaten.ElementAt(i).Gas) 
                 lRohdaten.ElementAt(i).MomentGas = lRohdaten.ElementAt(i + 1).Gas - lRohdaten.ElementAt(i).Gas;
                if (lRohdaten.ElementAt(i + 1).Wasser >= lRohdaten.ElementAt(i).Wasser)
                 lRohdaten.ElementAt(i).MomentWasser = lRohdaten.ElementAt(i + 1).Wasser - lRohdaten.ElementAt(i).Wasser;
            }
        }

        //------------------------------------------------------------
        // Momentdaten aus der Liste zu Statistikzwecken aufsummieren
        //------------------------------------------------------------
        float SummationStrom (uint UTCStart, uint UTCEnde)
        {
            float Temp = 0;
            foreach (var zaehlerdaten in lRohdaten)
            {
                if ((zaehlerdaten.UTC >= UTCStart) && (zaehlerdaten.UTC < UTCEnde))
                {
                    if (zaehlerdaten.MomentStrom < 0xFFFFFF) 
                    Temp += zaehlerdaten.MomentStrom;
                }
            }
            return (Temp);
        }

        //------------------------------------------------------------
        // Momentdaten aus der Liste zu Statistikzwecken aufsummieren
        //------------------------------------------------------------
        float SummationGas(uint UTCStart, uint UTCEnde)
        {
            float Temp = 0;
            foreach (var zaehlerdaten in lRohdaten)
            {
                if ((zaehlerdaten.UTC >= UTCStart) && (zaehlerdaten.UTC < UTCEnde))
                {
                    Temp += zaehlerdaten.MomentGas;
                }
            }
            return (Temp);
        }

        //------------------------------------------------------------
        // Momentdaten aus der Liste zu Statistikzwecken aufsummieren
        //------------------------------------------------------------
        float SummationWasser(uint UTCStart, uint UTCEnde)
        {
            float Temp = 0;
            foreach (var zaehlerdaten in lRohdaten)
            {
                if ((zaehlerdaten.UTC >= UTCStart) && (zaehlerdaten.UTC < UTCEnde))
                {
                    Temp += zaehlerdaten.MomentWasser;
                }
            }
            return (Temp);
        }


        //-------------------------------------
        // Zählerwerte zu Realwerten umrechnen
        //-------------------------------------
        void RealWertbestimmung ()
        {
            Strom /= 96; // 96 Umdrehungen entsprechen einer Kilowattstunde
            Gas /= 100;  // 1 Impuls entspricht 10l Gas, Umrechnung in Kubikmeter
            Wasser /= 1; // 1 Impuls = 1Liter, Anzeige Tagesverbrauch in Litern

        }

        //----------------------------
        // Summiert Grundlastwerte auf
        //----------------------------
        void GrundlastBerechnung(uint UTCStart, uint UTCEnde, int Zeitraum, uint GrundLastStart, uint GrundLastEnde)
        {
            
            uint start = UTCStart += GrundLastStart;
            uint ende = start + GrundLastEnde;    // 5 Uhr des Folgetages 

            Strom = 0;
            Gas = 0;
            for (uint i = 0; i < Zeitraum; i++)
            {
                uint Offset = (i * 24 * 60 * 60);
                Strom += SummationStrom(start + Offset, ende + Offset);
                Gas += SummationGas(start + Offset, ende + Offset);
            }

            Strom /= Zeitraum;
            Gas /= Zeitraum;
        }

        //------------------------------------------------
        // Anzahl der Tage im übergebenen Jahr berechnen
        //------------------------------------------------
        int GetDaysInAYear(int year)
        {

            int days = 0;
            for (int i = 1; i <= 12; i++)
            {
                days += DateTime.DaysInMonth(year, i);
            }
            return days;
        }

        //-----------------------------------
        // Statistik zur Datenbasis anzeigen
        //-----------------------------------
        void ZeigeDatenBasis()
        {
            LabelAnzahlDatensätze.Text = Convert.ToString(lRohdaten.Count);     // Wieviele Datensätze sind da ?

            // Von wann ist der erste Datensatz ?
            LabelStartDatenzeitraum.Text = lRohdaten.First().Datum;
            LabelStartDatenzeitraumUhrzeit.Text = lRohdaten.First().Uhrzeit + " Uhr";

            // Von wann der letzte Datensatz ?
            LabelEndeDatenZeitraum.Text = lRohdaten.Last().Datum;
            LabelEndeDatenZeitraumUhrzeit.Text = lRohdaten.Last().Uhrzeit + " Uhr";

            // Welcher Zeitraum wird beackerbar ? Durch die Furche rödel.......
            LabelDatenZeitraum.Text = Convert.ToString(lRohdaten.Count / (12 * 24)) + " Tage";

        }

        //------------------------------------------------------
        // Inhalte der Statistikseite errechnen und darstellen
        //------------------------------------------------------
        void ErrechneStatistikSeite()
        {
            //--------------
            // Heutiger Tag
            //--------------

            // Lokale Zeitvariable erzeugen
            System.DateTime localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());
            DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
            TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            uint StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);
            uint EndeUTC = lRohdaten.Last().UTC;

            // StartUTC enthält jetzt die UTC für den Beginn des heutigen Tages (z.B. 1300921200 = 24.3.2011 00:00 Uhr)

            // Gesamte Liste mit Rohdaten durchkämmen um den Zeitraum zu finden
            Strom = SummationStrom(StartUTC, EndeUTC);
            Gas = SummationGas(StartUTC, EndeUTC);
            Wasser = SummationWasser(StartUTC, EndeUTC);

            // Aktuelle Tagesverbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Tagesverbräuche auf Statistikseite ausgeben
            LabelStromVerbrauchHeute.Text = Strom.ToString("F1") + " kWh";
            LabelGasVerbrauchHeute.Text = Gas.ToString("F1") + " qm";
            LabelWasserverbrauchHeute.Text = Wasser.ToString("F0") + " Liter";

            //-------------------------
            // Gestrigen Tag ermitteln
            //-------------------------

            // Verbräuche gestern ermitteln

            // Lokale Zeitvariable erzeugen
            EndeUTC = StartUTC - 1;  // Start der Suche ist der Beginn des gestrigen Tages
            StartUTC -= 86400;          // Ende der Suche ist das Ende des gestrigen Tages

            // Gesamte Liste mit Rohdaten durchkämmen um den Zeitraum zu finden
            Strom = SummationStrom(StartUTC, EndeUTC);
            Gas = SummationGas(StartUTC, EndeUTC);
            Wasser = SummationWasser(StartUTC, EndeUTC);

            // Aktuelle Tagesverbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Gestrige Tagesverbräuche auf Statistikseite ausgeben
            LabelStromVerbrauchGestern.Text = Strom.ToString("F1") + " kWh";
            LabelGasVerbrauchGestern.Text = Gas.ToString("F1") + " qm";
            LabelWasserVerbrauchGestern.Text = Wasser.ToString("F0") + " Liter";

            //---------------------------------------------------
            // Grundlasten gestern ermitteln (Nachtverbrauch)
            //---------------------------------------------------


            StartUTC += GrundLastStart;            // 23 Uhr des gestrigen Tages 
            EndeUTC = StartUTC + GrundLastEnde;    // 5 Uhr des Folgetages 

            // Gesamte Liste mit Rohdaten durchkämmen um den Zeitraum zu finden
            Strom = SummationStrom(StartUTC, EndeUTC);
            Gas = SummationGas(StartUTC, EndeUTC);
            Wasser = SummationWasser(StartUTC, EndeUTC);

            // Aktuelle Tagesverbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Leistungsgberechnung vornehmen
            Strom /= ((24 - (GrundLastStart / 3600)) + (GrundLastEnde / 3600)); // Mache aus kWh kW
            Strom *= 1000;                                                      // Mache aus Kw W                

            // Gestrige Grundlasten auf Statistikseite ausgeben
            LabelStromGrundlastGestern.Text = Strom.ToString("F1") + " W";
            LabelGasGrundlastGestern.Text = Gas.ToString("F1") + " qm";

            //--------------------------
            // Wochenstatistik erzeugen
            //--------------------------
            int AnzahlWochentage = 1;
            while ((int)localDateTime.DayOfWeek != 1)
            {
                localDateTime = localDateTime.AddDays(-1);    // Lokalzeiger auf den letzten Montag setzen
                AnzahlWochentage++;                           // Anzahl vergangener Tage dieser Woche bestimmen  
            }
            ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn der aktuellen Woche
            EndeUTC = lRohdaten.Last().UTC;

            // Gesamte Liste mit Rohdaten durchkämmen um den Zeitraum zu finden
            Strom = SummationStrom(StartUTC, EndeUTC);
            Gas = SummationGas(StartUTC, EndeUTC);
            Wasser = SummationWasser(StartUTC, EndeUTC);

            // Aktuelle Tagesverbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Tagesverbräuche auf Statistikseite ausgeben
            LabelStromVerbrauchDieseWoche.Text = Strom.ToString("F1") + " kWh";
            LabelGasVerbrauchDieseWoche.Text = Gas.ToString("F1") + " qm";
            LabelWasserVerbrauchDieseWoche.Text = Wasser.ToString("F0") + " Liter";

            // GrundLasten dieser dieser Woche bestimmen
            GrundlastBerechnung(StartUTC, EndeUTC, AnzahlWochentage, GrundLastStart, GrundLastEnde);

            // Aktuelle Verbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Leistungsgberechnung vornehmen
            Strom /= ((24 - (GrundLastStart / 3600)) + (GrundLastEnde / 3600)); // Mache aus kWh kW
            Strom *= 1000;                                                      // Mache aus Kw W                

            // Gestrige Grundlasten auf Statistikseite ausgeben
            LabelStromGrundlastDieseWoche.Text = Strom.ToString("F1") + " W";
            LabelGasGrundlastDieseWoche.Text = Gas.ToString("F1") + " qm";

            //--------------------------------------------------
            // Statistische Daten für die letzte Woche erzeugen
            //--------------------------------------------------
            EndeUTC = StartUTC - 1;         // Ende der letzten Woche = Anfang dieser Woche-1
            localDateTime = localDateTime.AddDays(-7);      // 7 Tage Rückwärts
            ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn der letzten Woche
            EndeUTC = StartUTC + 7 * 24 * 60 * 60 - 1;               // Eine Woche -1 drauf = Wochenende

            // Gesamte Liste mit Rohdaten durchkämmen um den Zeitraum zu finden
            Strom = SummationStrom(StartUTC, EndeUTC);
            Gas = SummationGas(StartUTC, EndeUTC);
            Wasser = SummationWasser(StartUTC, EndeUTC);

            // Aktuelle Tagesverbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Tagesverbräuche auf Statistikseite ausgeben
            LabelStromVerbrauchLetzteWoche.Text = Strom.ToString("F1") + " kWh";
            LabelGasVerbrauchLetzteWoche.Text = Gas.ToString("F1") + " qm";
            LabelWasserVerbrauchLetzteWoche.Text = Wasser.ToString("F0") + " Liter";

            // GrundLasten dieser dieser Woche bestimmen
            GrundlastBerechnung(StartUTC, EndeUTC, 7, GrundLastStart, GrundLastEnde);

            // Aktuelle Verbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Leistungsgberechnung vornehmen
            Strom /= ((24 - (GrundLastStart / 3600)) + (GrundLastEnde / 3600)); // Mache aus kWh kW
            Strom *= 1000;                                                      // Mache aus Kw W                

            // Gestrige Grundlasten auf Statistikseite ausgeben
            LabelStromGrundlastLetzteWoche.Text = Strom.ToString("F1") + " W";
            LabelGasGrundlastLetzteWoche.Text = Gas.ToString("F1") + " qm";

            //-----------------------------
            // Monatsstatitiken errechnen
            //-----------------------------
            // Lokale Zeitvariable neu erzeugen auf aktuelles Datum
            localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());
            int Testmonat = localDateTime.Month;
            AnzahlWochentage = 1;
            // Tage bis zum Überschreiten der Monatsgrenze abziehen
            while (Testmonat == localDateTime.Month)
            {
                localDateTime = localDateTime.AddDays(-1);    // Lokalzeiger auf den letzten Montag setzen
                AnzahlWochentage++;                           // Anzahl vergangener Tage dieser Woche bestimmen  
            }
            // Verursachten Fehler wieder geradeziehen
            AnzahlWochentage--; localDateTime = localDateTime.AddDays(1);
            ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
            EndeUTC = lRohdaten.Last().UTC;

            // Gesamte Liste mit Rohdaten durchkämmen um den Zeitraum zu finden
            Strom = SummationStrom(StartUTC, EndeUTC);
            Gas = SummationGas(StartUTC, EndeUTC);
            Wasser = SummationWasser(StartUTC, EndeUTC);

            // Aktuelle Tagesverbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Aktuelle Monatsverbräuche auf Statistikseite ausgeben
            LabelStromVerbrauchDieserMonat.Text = Strom.ToString("F0") + " kWh";
            LabelGasVerbrauchDieserMonat.Text = Gas.ToString("F0") + " qm";
            LabelWasserVerbrauchDieserMonat.Text = Wasser.ToString("F0") + " Liter";

            // GrundLasten dieses Monats bestimmen
            GrundlastBerechnung(StartUTC, EndeUTC, AnzahlWochentage, GrundLastStart, GrundLastEnde);

            // Aktuelle Verbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Leistungsgberechnung vornehmen
            Strom /= ((24 - (GrundLastStart / 3600)) + (GrundLastEnde / 3600)); // Mache aus kWh kW
            Strom *= 1000;                                                      // Mache aus Kw W                

            // Diesmonatige Grundlasten auf Statistikseite ausgeben
            LabelStromGrundlastDieserMona.Text = Strom.ToString("F1") + " W";
            LabelGasGrundlastDieserMonat.Text = Gas.ToString("F1") + " qm";

            //----------------------------------------
            // Statistik des letzten Monats erzeugen
            //----------------------------------------
            localDateTime = localDateTime.AddMonths(-1);             // Einen Monat zurück
            AnzahlWochentage = System.DateTime.DaysInMonth(localDateTime.Year, localDateTime.Month);
            ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des vergangenen Monats
            int tage = DateTime.DaysInMonth(localDateTime.Year, localDateTime.Month); // Wie viele Tage hat besagter Monat ?
            EndeUTC = StartUTC + (uint)tage * 24 * 60 * 60 - 1;            // Ziegt jetzt auf das Ende des vergangen Monats

            // Gesamte Liste mit Rohdaten durchkämmen um den Zeitraum zu finden
            Strom = SummationStrom(StartUTC, EndeUTC);
            Gas = SummationGas(StartUTC, EndeUTC);
            Wasser = SummationWasser(StartUTC, EndeUTC);

            // Aktuelle Tagesverbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Letzte Monatsverbräuche auf Statistikseite ausgeben
            LabelStromVerbrauchLetzterMonat.Text = Strom.ToString("F0") + " kWh";
            LabelGasVerbrauchLetzterMonat.Text = Gas.ToString("F0") + " qm";
            LabelWasserVerbrauchLetzterMonat.Text = Wasser.ToString("F0") + " Liter";

            // GrundLasten des letzten Monats bestimmen
            GrundlastBerechnung(StartUTC, EndeUTC, AnzahlWochentage, GrundLastStart, GrundLastEnde);

            // Aktuelle Verbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Leistungsgberechnung vornehmen
            Strom /= ((24 - (GrundLastStart / 3600)) + (GrundLastEnde / 3600)); // Mache aus kWh kW
            Strom *= 1000;                                                      // Mache aus Kw W                

            // Letztmonatige Grundlasten auf Statistikseite ausgeben
            LabelStromGrundlastLetzterMonat.Text = Strom.ToString("F1") + " W";
            LabelGasGrundlastLetzterMonat.Text = Gas.ToString("F1") + " qm";

            //--------------------------
            // Aktuelles Jahr beackern
            //--------------------------
            localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());  // Datum wieder geradeziehen         
            int AnzahlTageimJahr = localDateTime.DayOfYear;                                     // Anzahl der vergangenen Tage in diesem Jahr
            int Jahr = localDateTime.Year;
            localDateTime = new DateTime(Jahr, 1, 1);                                           // Referenzdatum = Beginn des Jahres

            ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
            EndeUTC = lRohdaten.Last().UTC;

            // Gesamte Liste mit Rohdaten durchkämmen um den Zeitraum zu finden
            Strom = SummationStrom(StartUTC, EndeUTC);
            Gas = SummationGas(StartUTC, EndeUTC);
            Wasser = SummationWasser(StartUTC, EndeUTC);

            // Aktuelle Tagesverbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung(); Wasser /= 1000;

            // Aktuelle Monatsverbräuche auf Statistikseite ausgeben
            LabelStromVerbrauchDiesesJahr.Text = Strom.ToString("F0") + " kWh";
            LabelGasVerbrauchDiesesJahr.Text = Gas.ToString("F0") + " qm";
            LabelWasserVerbrauchDiesesJahr.Text = Wasser.ToString("F0") + " qm";

            // GrundLasten dieses Monats bestimmen
            GrundlastBerechnung(StartUTC, EndeUTC, AnzahlTageimJahr, GrundLastStart, GrundLastEnde);

            // Aktuelle Verbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Leistungsgberechnung vornehmen
            Strom /= ((24 - (GrundLastStart / 3600)) + (GrundLastEnde / 3600)); // Mache aus kWh kW
            Strom *= 1000;                                                      // Mache aus Kw W                

            // Diesjährige Grundlasten auf Statistikseite ausgeben
            LabelStromGrundlastDiesesJahr.Text = Strom.ToString("F1") + " W";
            LabelGasGrundlastDiesesJahr.Text = Gas.ToString("F1") + " qm";

            //------------------------
            // Letztes Jahr errechnen
            //------------------------
            localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());  // Datum wieder geradeziehen         
            Jahr = localDateTime.Year;
            localDateTime = new DateTime(Jahr - 1, 1, 1);                                        // Referenzdatum = Beginn des letzten Jahres
            AnzahlTageimJahr = GetDaysInAYear(Jahr - 1);
            ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
            EndeUTC = StartUTC + (uint)AnzahlTageimJahr * 24 * 60 * 60;

            // Gesamte Liste mit Rohdaten durchkämmen um den Zeitraum zu finden
            Strom = SummationStrom(StartUTC, EndeUTC);
            Gas = SummationGas(StartUTC, EndeUTC);
            Wasser = SummationWasser(StartUTC, EndeUTC);

            // Aktuelle Tagesverbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung(); Wasser /= 1000;

            // Aktuelle Monatsverbräuche auf Statistikseite ausgeben
            LabelStromVerbrauchLetztesJahr.Text = Strom.ToString("F0") + " kWh";
            LabelGasVerbrauchLetztesJahr.Text = Gas.ToString("F0") + " qm";
            LabelWasserVerbrauchLetztesJahr.Text = Wasser.ToString("F0") + " qm";

            // GrundLasten des letzten Jahres bestimmen
            GrundlastBerechnung(StartUTC, EndeUTC, AnzahlTageimJahr, GrundLastStart, GrundLastEnde);

            // Aktuelle Verbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Leistungsgberechnung vornehmen
            Strom /= ((24 - (GrundLastStart / 3600)) + (GrundLastEnde / 3600)); // Mache aus kWh kW
            Strom *= 1000;                                                      // Mache aus Kw W                

            // Letztjährige Grundlasten auf Statistikseite ausgeben
            LabelStromGrundlastLetztesJahr.Text = Strom.ToString("F1") + " W";
            LabelGasGrundlastLetztesJahr.Text = Gas.ToString("F1") + " qm";

            //---------------------------------
            // Abrechnungszeitraum darstellen
            //---------------------------------

            // Als erstes Strom
            //------------------
            System.DateTime dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumStrom);
            // localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());  // Datum wieder geradeziehen         
            ts = new TimeSpan(dateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
            EndeUTC = lRohdaten.Last().UTC;

            // Gesamte Liste mit Rohdaten durchkämmen um den Zeitraum zu finden
            Strom = SummationStrom(StartUTC, EndeUTC);

            // Aktuelle Tagesverbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung(); // Wasser /= 1000;

            // Aktuellen Abrechnungsverbauch strom auf Statistikseite ausgeben
            LabelAbrechnungszeitraumStrom.Text = Strom.ToString("F0") + " kWh";

            // GrundLasten dieses Monats bestimmen
            AnzahlTageimJahr = (int)((EndeUTC - StartUTC) / (60 * 60 * 24));
            GrundlastBerechnung(StartUTC, EndeUTC, AnzahlTageimJahr, GrundLastStart, GrundLastEnde);

            // Aktuelle Verbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            // Leistungsgberechnung vornehmen
            Strom /= ((24 - (GrundLastStart / 3600)) + (GrundLastEnde / 3600)); // Mache aus kWh kW
            Strom *= 1000;                                                      // Mache aus Kw W                

            // Diesjährige Grundlasten auf Statistikseite ausgeben
            LabelAbrechnungszeitraumStromGrundlast.Text = Strom.ToString("F1") + " W";

            // Abrechnungszeitraum Gas
            //--------------------------
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumGas);
            // localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());  // Datum wieder geradeziehen         
            ts = new TimeSpan(dateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
            EndeUTC = lRohdaten.Last().UTC;
            Gas = SummationGas(StartUTC, EndeUTC);
            RealWertbestimmung();

            LabelAbrechnungszeitraumGas.Text = Gas.ToString("F0") + " qm";
            GrundlastBerechnung(StartUTC, EndeUTC, AnzahlTageimJahr, GrundLastStart, GrundLastEnde);

            // Aktuelle Verbräuche bestimmen und mit Zählerkonstanten verrechnen
            RealWertbestimmung();

            LabelAbrechnungszeitraumGasGrundlast.Text = Gas.ToString("F1") + " qm";
            
            // Abrechnungszeitraum Wasser
            //-----------------------------
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumWasser);
            ts = new TimeSpan(dateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
            EndeUTC = lRohdaten.Last().UTC;
            Wasser = SummationWasser(StartUTC, EndeUTC);
            Wasser /= 1000;
            LabelAbrechnungszeitraumWasser.Text = Wasser.ToString("F0") + " qm";

            //---------------------------------
            // Aktuelle Zählerstände ausgeben
            //---------------------------------
            double StandStrom = 44599.870F + (((double)lRohdaten.Last().Strom) / 96);
            LabelzaehlerstandStrm.Text = StandStrom.ToString("F1") + " kWh";

            double StandGas = 43378.25F + (((double)lRohdaten.Last().Gas) / 100);
            LabelzaehlerstandGas.Text = StandGas.ToString("F3") + " qm";      
            
            double StandWasser = 353.599F + (((double)lRohdaten.Last().Wasser) / 1000);
            LabelzaehlerstandWasser.Text = StandWasser.ToString("F3") + " qm";

            //-----------------------
            // Kontostände errechnen
            //-----------------------
            
            // Als erstes Strom
            //------------------
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumStrom);
            // if (dateTime.Day > 1) dateTime = dateTime.AddMonths(1);      // Einen Monat weiter falls wir nicht am Ersten tag des Monats beginnen
            ts = new TimeSpan(dateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
            EndeUTC = lRohdaten.Last().UTC;
            AnzahlTageimJahr = (int)((EndeUTC - StartUTC) / (60 * 60 * 24));

            float AufgelaufeneKosten = 0;

            float Temp = 0;
            int Abschlagszeiger = 0;
            float Zahlungen = 0;
            foreach (var zaehlerdaten in lRohdaten)
            {

                //------------------------------------------------------
                // Befinden wir uns innerhalb des gültigen Zeitraumes ?
                //------------------------------------------------------
                if (zaehlerdaten.UTC >= StartUTC)
                {   // Zeitraum ist gültig

                    // Liste mit den Abschlägen durchgehen und den passenden für den aktuellen Zeitraum raussuchen
                    Abschlagszeiger = 0;
                    foreach (var kosten in lKostendaten)
                    {
                        if (kosten.DatumUTC > zaehlerdaten.UTC) break;
                        Abschlagszeiger++;
                        if (Abschlagszeiger == lKostendaten.Count) { Abschlagszeiger = lKostendaten.Count - 1; break; }; 
                    } // Von foreach (var Abschlaege in lAbschlaege)
                    Temp += ((float)zaehlerdaten.MomentStrom / 96) * lKostendaten.ElementAt(Abschlagszeiger).StromArbeitspreis;
                    // Temp += zaehlerdaten.MomentStrom;                
                }


            } // Von foreach (var zaehlerdaten in lRohdaten)
            Temp += (lKostendaten.ElementAt(Abschlagszeiger).StromGrundpreis / 365) * AnzahlTageimJahr;
            // Temp enthält jetzt die im Abrechnungszeitraum aufgelaufenen Kosten, diese werden mit den Abschlägen verrechnet
            Zahlungen = 0;
            // Liste mit den Abschlägen durchgehen
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumStrom);

            while (dateTime < System.DateTime.Now)
            {
                Abschlagszeiger = 0;
                foreach (var abschlaege in lAbschlaege)
                {
                    ts = new TimeSpan(dateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                    uint Date = (uint)Convert.ToInt32(ts.TotalSeconds);
                        // Date = gerade beackerters Monatsdatum
                    if (abschlaege.Datum > Date) { if (Abschlagszeiger > 0) Abschlagszeiger--; break;  };
                    Abschlagszeiger++;
                    if (Abschlagszeiger == lAbschlaege.Count) { Abschlagszeiger = lAbschlaege.Count - 1; break; }; 

                } // Von foreach (var Abschlaege in lAbschlaege)
                Zahlungen += lAbschlaege.ElementAt(Abschlagszeiger).Strom;  // Aktuell ermittelten Abschlag dazuballern
                dateTime = dateTime.AddMonths(1);      // Einen Monat weiter
                
            } // Von while (dateTime < System.DateTime.Now)
            // Zahlungen enthält jetzt die Summe der Abschläge uber den gewählten Zeitraum
            AufgelaufeneKosten = Zahlungen - Temp;
            if (AufgelaufeneKosten < 0) LabelKontostandStrom.ForeColor = Color.Red; else
                                        LabelKontostandStrom.ForeColor = Color.Black; 

            LabelKontostandStrom.Text = AufgelaufeneKosten.ToString("F2") + " €"; //  +Zahlungen.ToString("F2") + (" €");

            //------------
            // Jetzt Gas
            //------------
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumGas);
            // if (dateTime.Day > 1) dateTime = dateTime.AddMonths(1);      // Einen Monat weiter falls wir nicht am Ersten tag des Monats beginnen

            ts = new TimeSpan(dateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
            EndeUTC = lRohdaten.Last().UTC;
            AnzahlTageimJahr = (int)((EndeUTC - StartUTC) / (60 * 60 * 24));

            AufgelaufeneKosten = 0;
            Temp = 0;
            Abschlagszeiger = 0;
            Zahlungen = 0;
            foreach (var zaehlerdaten in lRohdaten)
            {

                //------------------------------------------------------
                // Befinden wir uns innerhalb des gültigen Zeitraumes ?
                //------------------------------------------------------
                if (zaehlerdaten.UTC >= StartUTC)
                {   // Zeitraum ist gültig

                    // Liste mit den Abschlägen durchgehen und den passenden für den aktuellen Zeitraum raussuchen
                    Abschlagszeiger = 0;
                    foreach (var kosten in lKostendaten)
                    {
                        if (kosten.DatumUTC > zaehlerdaten.UTC) break;
                        Abschlagszeiger++;
                        if (Abschlagszeiger == lKostendaten.Count) { Abschlagszeiger = lKostendaten.Count - 1; break; };
                    } // Von foreach (var Abschlaege in lAbschlaege)
                    Temp += ((float)zaehlerdaten.MomentGas / 100) * lKostendaten.ElementAt(Abschlagszeiger).Gararbeitspreis;
                    // Temp += zaehlerdaten.MomentStrom;                
                }


            } // Von foreach (var zaehlerdaten in lRohdaten)
            Temp += (lKostendaten.ElementAt(Abschlagszeiger).GasGrundpreis / 365) * AnzahlTageimJahr;
            // Temp enthält jetzt die im Abrechnungszeitraum aufgelaufenen Kosten, diese werden mit den Abschlägen verrechnet
            Zahlungen = 0;
            // Liste mit den Abschlägen durchgehen
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumGas);

            while (dateTime < System.DateTime.Now)
            {
                Abschlagszeiger = 0;
                foreach (var abschlaege in lAbschlaege)
                {
                    ts = new TimeSpan(dateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                    uint Date = (uint)Convert.ToInt32(ts.TotalSeconds);
                    // Date = gerade beackerters Monatsdatum
                    if (abschlaege.Datum > Date) { if (Abschlagszeiger > 0) Abschlagszeiger--; break; };
                    Abschlagszeiger++;
                    if (Abschlagszeiger == lAbschlaege.Count) { Abschlagszeiger = lAbschlaege.Count - 1; break; };

                } // Von foreach (var Abschlaege in lAbschlaege)
                Zahlungen += lAbschlaege.ElementAt(Abschlagszeiger).Gas;  // Aktuell ermittelten Abschlag dazuballern
                dateTime = dateTime.AddMonths(1);      // Einen Monat weiter

            } // Von while (dateTime < System.DateTime.Now)
            // Zahlungen enthält jetzt die Summe der Abschläge uber den gewählten Zeitraum
            AufgelaufeneKosten = Zahlungen - Temp;
            if (AufgelaufeneKosten < 0) LabelKontostandGas.ForeColor = Color.Red; else
                                        LabelKontostandGas.ForeColor = Color.Black; 

            LabelKontostandGas.Text = AufgelaufeneKosten.ToString("F2") + " €"; //  +Zahlungen.ToString("F2") + (" €");

            //------------------------
            // Und Wasser zum Schluss
            //------------------------
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumWasser);
            // if (dateTime.Day > 1) dateTime = dateTime.AddMonths(1);      // Einen Monat weiter falls wir nicht am Ersten tag des Monats beginnen

            ts = new TimeSpan(dateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            StartUTC = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
            EndeUTC = lRohdaten.Last().UTC;
            AnzahlTageimJahr = (int)((EndeUTC - StartUTC) / (60 * 60 * 24));

            AufgelaufeneKosten = 0;
            Temp = 0;
            Abschlagszeiger = 0;
            Zahlungen = 0;
            foreach (var zaehlerdaten in lRohdaten)
            {

                //------------------------------------------------------
                // Befinden wir uns innerhalb des gültigen Zeitraumes ?
                //------------------------------------------------------
                if (zaehlerdaten.UTC >= StartUTC)
                {   // Zeitraum ist gültig

                    // Liste mit den Abschlägen durchgehen und den passenden für den aktuellen Zeitraum raussuchen
                    Abschlagszeiger = 0;
                    foreach (var kosten in lKostendaten)
                    {
                        if (kosten.DatumUTC > zaehlerdaten.UTC) break;
                        Abschlagszeiger++;
                        if (Abschlagszeiger == lKostendaten.Count) { Abschlagszeiger = lKostendaten.Count - 1; break; };
                    } // Von foreach (var Abschlaege in lAbschlaege)
                    Temp += ((float)zaehlerdaten.MomentWasser / 1000) * lKostendaten.ElementAt(Abschlagszeiger).WasserArbeitspreis;
                    // Temp += zaehlerdaten.MomentStrom;                
                }


            } // Von foreach (var zaehlerdaten in lRohdaten)
            Temp += (lKostendaten.ElementAt(Abschlagszeiger).WasserGrundpreis/ 365) * AnzahlTageimJahr;
            // Temp enthält jetzt die im Abrechnungszeitraum aufgelaufenen Kosten, diese werden mit den Abschlägen verrechnet
            Zahlungen = 0;
            // Liste mit den Abschlägen durchgehen
            dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
            
            // dateTime = dateTime.AddSeconds(lAbschlaege.First().Datum);
            dateTime = dateTime.AddSeconds(lAblesedaten.Last().DatumWasser);
            // kagge
            while (dateTime < System.DateTime.Now)
            {
                Abschlagszeiger = 0;
                foreach (var abschlaege in lAbschlaege)
                {
                    ts = new TimeSpan(dateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                    uint Date = (uint)Convert.ToInt32(ts.TotalSeconds);
                    // Date = gerade beackerters Monatsdatum
                    if (abschlaege.Datum > Date) { if (Abschlagszeiger > 0) Abschlagszeiger--; break; };
                    Abschlagszeiger++;
                    if (Abschlagszeiger == lAbschlaege.Count) { Abschlagszeiger = lAbschlaege.Count - 1; break; };

                } // Von foreach (var Abschlaege in lAbschlaege)
                Zahlungen += lAbschlaege.ElementAt(Abschlagszeiger).Wasser;  // Aktuell ermittelten Abschlag dazuballern
                dateTime = dateTime.AddMonths(1);      // Einen Monat weiter

            } // Von while (dateTime < System.DateTime.Now)
            // Zahlungen enthält jetzt die Summe der Abschläge uber den gewählten Zeitraum
            AufgelaufeneKosten = Zahlungen - Temp;
            if (AufgelaufeneKosten < 0) LabelKontostandWasser.ForeColor = Color.Red; else
                                        LabelKontostandWasser.ForeColor = Color.Black; 
            LabelKontostandWasser.Text = AufgelaufeneKosten.ToString("F2") + " €"; //  +Zahlungen.ToString("F2") + (" €");

        }

        //--------------------------
        // Grafikausgabe anreissen
        //--------------------------
        void InitGrafik()
        {
            DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
            
            zedGraphControl1.IsAntiAlias = true;

            Grafik = zedGraphControl1.GraphPane;

            // Achsenbeschriftung
            Grafik.Title.Text = "Ressourcenmonitor";
            Grafik.XAxis.Title.Text = "Zeitraum";
            Grafik.YAxis.Title.Text = "Strom";
            Grafik.Y2Axis.Title.Text = "Gas/Wasser";

            Grafik.YAxis.Title.FontSpec.FontColor = Color.Blue;
            Grafik.Y2Axis.Title.FontSpec.FontColor = Color.Red;

            // Dritte Y-Achse erzeugen
            // penis YAxis yAxis3 = new YAxis("Temperatur [°C]");
            Grafik.YAxisList.Add(yAxis3);
            yAxis3.Scale.FontSpec.FontColor = Color.Green;
            yAxis3.Title.FontSpec.FontColor = Color.Green;
            yAxis3.Color = Color.Green;
            // turn off the opposite tics so the Y2 tics don't show up on the Y axis
            yAxis3.MajorTic.IsInside = false;
            yAxis3.MinorTic.IsInside = false;
            yAxis3.MajorTic.IsOpposite = false;
            yAxis3.MinorTic.IsOpposite = false;
            // Align the Y2 axis labels so they are flush to the axis
            yAxis3.Scale.Align = AlignP.Inside;
            yAxis3.IsVisible = false;                           // Y-Achse ist zunächst unsichtbar

            // Kurvendaten
            Stromkurvendaten = new PointPairList();
            Gaskurvendaten = new PointPairList();
            Wasserkurvendaten = new PointPairList();
            Grundlastkurvendaten = new PointPairList();
            GrundlastKurvenDatenGlatt = new PointPairList();

            KurveAussen = new PointPairList();
            KurveHeizungVorlauf = new PointPairList();
            KurveHeizungRuecklauf = new PointPairList();
            KurveBoilerEinlass = new PointPairList();
            KurveBoilerAuslass = new PointPairList();
            KurveAbgasHeizung = new PointPairList();
            KurveAbgasBoiler = new PointPairList();
            KurveHeizungsraum = new PointPairList();
  
            // X-Achse zeigt Datum
            Grafik.XAxis.Type = AxisType.Date;

            LineItem Stromkurve = Grafik.AddCurve("Strom[kW]", Stromkurvendaten, Color.Blue, SymbolType.None);
            LineItem Gaskurve = Grafik.AddCurve("Gas[l]", Gaskurvendaten, Color.Red, SymbolType.None);
            LineItem Wasserkurve = Grafik.AddCurve("Wasser[l]", Wasserkurvendaten, Color.Magenta, SymbolType.Circle);
            LineItem Grundlastkurve = Grafik.AddCurve("Grundlast[kw]", Grundlastkurvendaten, Color.Blue, SymbolType.Star);
            LineItem GrundlastkurveGlatt = Grafik.AddCurve("Grundlast int.[kW]", GrundlastKurvenDatenGlatt, Color.Black, SymbolType.Circle);

            // Temperaturkurven zusammenbauen Mistkacke
            LineItem TKurveAussen = Grafik.AddCurve("", KurveAussen, Color.Green, SymbolType.None);
            LineItem TKurveHeizungVorlauf = Grafik.AddCurve("", KurveHeizungVorlauf, Color.Green, SymbolType.None);
            LineItem TKurveHeizungRuecklauf = Grafik.AddCurve("", KurveHeizungRuecklauf, Color.Green, SymbolType.None);
            LineItem TKurveBoilerEinlass = Grafik.AddCurve("", KurveBoilerEinlass, Color.Green, SymbolType.None);
            LineItem TKurveBoilerAuslass = Grafik.AddCurve("", KurveBoilerAuslass, Color.Green, SymbolType.None);
            LineItem TKurveAbgasHeizung = Grafik.AddCurve("", KurveAbgasHeizung, Color.Green, SymbolType.None);
            LineItem TKurveAbgasBoiler = Grafik.AddCurve("", KurveAbgasBoiler, Color.Green, SymbolType.None);
            LineItem TKurveHeizungsraum = Grafik.AddCurve("", KurveHeizungsraum, Color.Green, SymbolType.None);

            // Linienbreite
            Stromkurve.Line.Width = 2.0F;
            Gaskurve.Line.Width = 2.0F;
            Wasserkurve.Line.Width = 2.0F;
            Grundlastkurve.Line.Width = 2.0F;
            GrundlastkurveGlatt.Line.Width = 2.0F;

            // Gas und Wasser bekommen eine eigene Y-Achse
            Gaskurve.IsY2Axis = true;
            Wasserkurve.IsY2Axis = true;

            // Grafik.XAxis.MajorGrid.IsVisible = true;

            Grafik.YAxis.Title.FontSpec.FontColor = Color.Blue;
            // turn off the opposite tics so the Y tics don't show up on the Y2 axis
            Grafik.YAxis.MajorTic.IsOpposite = false;
            // Don't display the Y zero line
            Grafik.YAxis.MajorGrid.IsZeroLine = false;
           // Align the Y axis labels so they are flush to the axis
            Grafik.YAxis.Scale.Align = AlignP.Inside;
             
            Grafik.Y2Axis.IsVisible = false;
            Grafik.Y2Axis.Scale.FontSpec.FontColor = Color.Red;
            
            // turn off the opposite tics so the Y2 tics don't show up on the Y axis
            Grafik.Y2Axis.MajorTic.IsOpposite = false;
            Grafik.Y2Axis.MinorTic.IsOpposite = false;

            // Display the Y2 axis grid lines
           // Align the Y2 axis labels so they are flush to the axis
            Grafik.Y2Axis.Scale.Align = AlignP.Inside;

            //----------------------------------
            // Kurven erzeugen aus den Rohdaten
            //----------------------------------
            int i = 0;
            foreach (var zaehlerdaten in lRohdaten)
            {
                double x,y;
               
                // Datum erzeugen
                DateTime Zeitpunkt = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
                Zeitpunkt = Zeitpunkt.AddSeconds (zaehlerdaten.UTC);
                x = (double)new XDate(Zeitpunkt.Year, Zeitpunkt.Month, Zeitpunkt.Day, Zeitpunkt.Hour, Zeitpunkt.Minute, Zeitpunkt.Second);

                // Erstes Element
                if (i == 0)
                {
                    TimeSpan ts = new TimeSpan(Zeitpunkt.Ticks - RefDate.Ticks);  // das Delta ermitteln
                    GrafikXStart = (uint)ts.TotalSeconds;
                    //LabelGrafikStart.Text = Zeitpunkt.ToShortDateString();
                    // dateTimePicker1.Value = Zeitpunkt;
                }
                // Letzte Element
                if (i == lRohdaten.Count-1)
                {
                    TimeSpan ts = new TimeSpan(Zeitpunkt.Ticks - RefDate.Ticks);  // das Delta ermitteln
                    GrafikXEnde = (uint)ts.TotalSeconds;
                    //LabelGrafikEnde.Text = Zeitpunkt.ToShortDateString();
                    // dateTimePicker2.Value = Zeitpunkt;
                }

                // Kurvendaten erzeugen Scheisse
                i++;   
                // Stromdaten in kw umrechnen
                y = zaehlerdaten.MomentStrom;
                y /= 96;
                y *= 12;
 
                Stromkurvendaten.Add(x, y);     // Anzeige in kW 
                
                // Uhrzeit zur Berechnung der Grundlasten heranziehen
                if ( ((Zeitpunkt.Hour*3600 + Zeitpunkt.Minute*60) > GrundLastEnde) &&
                     ((Zeitpunkt.Hour*3600 + Zeitpunkt.Minute*60) < GrundLastStart) ) y = 0;
             
                Grundlastkurvendaten.Add(x, y); // Anzeige in kW 
                GrundlastKurvenDatenGlatt.Add(x, y);
       
                Gaskurvendaten.Add(x, zaehlerdaten.MomentGas*10);      // Anzeige in Litern Gas
                Wasserkurvendaten.Add(x, zaehlerdaten.MomentWasser);   // Anzeige in Litern Wasser 

                // Temperaturkurven zusammenbauen
                y = zaehlerdaten.Temperatur1;
                KurveAussen.Add(x, y);              // Anzeige in °C

                y = zaehlerdaten.Temperatur2;
                KurveHeizungVorlauf.Add(x, y);     // Anzeige in °C

                y = zaehlerdaten.Temperatur3;
                KurveHeizungRuecklauf.Add(x, y);     // Anzeige in °C

                y = zaehlerdaten.Temperatur4;
                KurveBoilerEinlass.Add(x, y);     // Anzeige in °C

                y = zaehlerdaten.Temperatur5;
                KurveBoilerAuslass.Add(x, y);     // Anzeige in °C

                y = zaehlerdaten.Temperatur6;
                KurveAbgasHeizung.Add(x, y);     // Anzeige in °C

                y = zaehlerdaten.Temperatur7;
                KurveAbgasBoiler.Add(x, y);     // Anzeige in °C

                y = zaehlerdaten.Temperatur8;
                KurveHeizungsraum.Add(x, y);     // Anzeige in °C
                
            }

            //------------------------------------------------------------------
            // Geglättete Grundlastkurve erzeugen durch schwimmenden Mittelwert
            //------------------------------------------------------------------
            int index = 0;          // double xw, yw;
            int Delta = 10;          // Immer 3 Werte berücksichtigen
            double Summe = 0;
            int j;
            
            for (index = 0; index < lRohdaten.Count-Delta; index++)
            {
                DateTime Zeitpunkt2 = new DateTime(1970, 1, 1); 
                Zeitpunkt2 = XDate.XLDateToDateTime(Grundlastkurvendaten.ElementAt(index).X);
                Summe = 0;
                for (j = index; j < index+Delta; j++)
                {
                    Summe += Grundlastkurvendaten.ElementAt(j).Y;
                }
                Summe /= Delta;
                // Uhrzeit zur Berechnung der Grundlasten heranziehen
                if (((Zeitpunkt2.Hour * 3600 + Zeitpunkt2.Minute * 60) > GrundLastEnde) &&
                 ((Zeitpunkt2.Hour * 3600 + Zeitpunkt2.Minute * 60) < GrundLastStart)) Summe = 0;

                GrundlastKurvenDatenGlatt.ElementAt(index).Y = Summe;
            }

            // Kurven ein- oder ausgeschaltet ?
            Grafik.CurveList[0].IsVisible = GrafikStrom;
            Grafik.CurveList[1].IsVisible = GrafikGas;
            Grafik.CurveList[2].IsVisible = GrafikWasser;

            Grafik.CurveList[3].IsVisible = false;
            Grafik.CurveList[4].IsVisible = false;
            if (GrafikGrundlast == 1) Grafik.CurveList[3].IsVisible = true; // Grundlast
            if (GrafikGrundlast == 2) Grafik.CurveList[4].IsVisible = true; // Geglättete Grundlast

            // Bezüge, Kurve zu Y-Achse
            Grafik.CurveList[0].IsY2Axis = false;   // Strom, Bezug ist die Achse links
            Grafik.CurveList[1].IsY2Axis = true;   // Gas, bezug ist die Achse rechts
            Grafik.CurveList[2].IsY2Axis = true;   // Wasser, bezug ist die Achse rechts
            Grafik.CurveList[3].IsY2Axis = false;   // Grundlast, Bezug ist die Achse links
            Grafik.CurveList[4].IsY2Axis = false;   // Grundlast Glatt, Bezug ist die Achse links

            // Grafik.CurveList[4].YAxisIndex 

            // Alle temperaturkurven sind der dritten Y-Achse zugeordnet
            /*
            Grafik.CurveList[5].YAxisIndex = 1;
            Grafik.CurveList[6].YAxisIndex = 1;
            Grafik.CurveList[7].YAxisIndex = 1;
            Grafik.CurveList[8].YAxisIndex = 1;
            Grafik.CurveList[9].YAxisIndex = 1;
            Grafik.CurveList[10].YAxisIndex = 1;
            Grafik.CurveList[11].YAxisIndex = 1;
            Grafik.CurveList[12].YAxisIndex = 1;
            */

            // Temperturkurven zunächst unsichtbar
            Grafik.CurveList[5].IsVisible = false;
            Grafik.CurveList[6].IsVisible = false;
            Grafik.CurveList[7].IsVisible = false;
            Grafik.CurveList[8].IsVisible = false;
            Grafik.CurveList[9].IsVisible = false;
            Grafik.CurveList[10].IsVisible = false;
            Grafik.CurveList[11].IsVisible = false;
            Grafik.CurveList[12].IsVisible = false;
            
            // Maxima ermitteln
            StromMax = Stromkurvendaten.Max(y => y.Y); // 5.125
            GasMax = Gaskurvendaten.Max(y => y.Y);    // 1130
            WasserMax = Wasserkurvendaten.Max(y => y.Y); // 65
            GrundlastMax = Grundlastkurvendaten.Max(y => y.Y); // 5.125

            //Automatische Wahl des Formates aktivieren (abhängig vom Zoombereich)
            Grafik.XAxis.Scale.FormatAuto = true;
            //Einteilung der X-Achse automatisch festlegen
            Grafik.XAxis.Scale.MajorStepAuto = true;
            Grafik.XAxis.Scale.MinorStepAuto = true;

            Grafik.YAxis.Scale.MajorStepAuto = true;
            Grafik.YAxis.Scale.MinorStepAuto = true;
            
            Grafik.Y2Axis.Scale.MajorStepAuto = true;
            Grafik.Y2Axis.Scale.MinorStepAuto = true;

            yAxis3.Scale.MajorStepAuto = true;
            yAxis3.Scale.MinorStepAuto = true;

            // Dritte Y-Achse

            // yAxis3.Scale.MinorStep = 10;
            yAxis3.Scale.Min = -50;
            yAxis3.Scale.Max = 150;
            
            // Hintergrund anhübschen

            Grafik.Chart.Fill = new Fill(Color.White, Color.LightGoldenrodYellow, 45.0f);
            //Graph01.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;

            Grafik.YAxis.MajorGrid.IsVisible = true;

            Grafik.AxisChange();
            
            // Grafik.IsAntiAlias = false;
            

        }

        //------------------------------
        // Rotzt ne schöne Grafik raus
        //------------------------------
        void ErzeugeGrafik()
        {
           
            // Kurven ein- oder ausgeschaltet ?
            Grafik.CurveList[0].IsVisible = GrafikStrom;
            Grafik.CurveList[1].IsVisible = GrafikGas;
            Grafik.CurveList[2].IsVisible = GrafikWasser;
            
            Grafik.CurveList[3].IsVisible = false;
            Grafik.CurveList[4].IsVisible = false;
            if (GrafikGrundlast == 1) Grafik.CurveList[3].IsVisible = true; // Grundlast
            if (GrafikGrundlast == 2) Grafik.CurveList[4].IsVisible = true; // Geglättete Grundlast

            // Y-Achsen wegschalten wenn nicht gebraucht
            if ((!GrafikStrom) && (GrafikGrundlast == 0))
            {
                Grafik.YAxis.MajorGrid.IsVisible = false;
                Grafik.YAxis.IsVisible = false;

            }
            else
            {
                Grafik.YAxis.MajorGrid.IsVisible = true;
                Grafik.YAxis.IsVisible = true;
            }

            if ((!GrafikGas) && (!GrafikWasser))
            {
                Grafik.Y2Axis.IsVisible = false;
                Grafik.Y2Axis.MajorGrid.IsVisible = false;
            }
            else
            {
                Grafik.Y2Axis.IsVisible = true;
                Grafik.Y2Axis.MajorGrid.IsVisible = true;
            }

            // Skalierung testweise anpassen scheisse
            Grafik.XAxis.Scale.MajorStep = (GrafikXEnde - GrafikXStart) / 10;

            // Neuscalierung der Y-Achsen durchführen
            Grafik.YAxis.Scale.Min = 0;

            float Skalenfaktor = 1.1F;

            // Erste Y-Achse
            //----------------
            
            // Strom und Grundlast normal
            if (GrafikStrom && GrafikGrundlast==1)
                if (Stromkurvendaten.Max(y => y.Y) > Grundlastkurvendaten.Max(y => y.Y))
                    Grafik.YAxis.Scale.Max = Stromkurvendaten.Max(y => y.Y) * Skalenfaktor; else
                    Grafik.YAxis.Scale.Max = Grundlastkurvendaten.Max(y => y.Y) * Skalenfaktor;

            // Strom und Grundlast glatt 
            if (GrafikStrom && GrafikGrundlast == 1)
                if (Stromkurvendaten.Max(y => y.Y) > GrundlastKurvenDatenGlatt.Max(y => y.Y))
                    Grafik.YAxis.Scale.Max = Stromkurvendaten.Max(y => y.Y) * Skalenfaktor;  else
                    Grafik.YAxis.Scale.Max = GrundlastKurvenDatenGlatt.Max(y => y.Y) * Skalenfaktor; 

            // Strom, keine Grundlast
            if (GrafikStrom && GrafikGrundlast==0)
                Grafik.YAxis.Scale.Max = Stromkurvendaten.Max(y => y.Y) * Skalenfaktor;

            // Kein Strom, Normale Grundlast
            if (!GrafikStrom && GrafikGrundlast==1)
                Grafik.YAxis.Scale.Max = Grundlastkurvendaten.Max(y => y.Y) * Skalenfaktor;
          
            // Kein Strom, Glatte Grundlast
            if (!GrafikStrom && GrafikGrundlast == 2)
            {
                Grafik.YAxis.Scale.Max = GrundlastKurvenDatenGlatt.Max(y => y.Y) * Skalenfaktor;
                double scale = GrundlastKurvenDatenGlatt.Max(y => y.Y) * Skalenfaktor;
                
            }

            // Zweite Y-Achse
            //----------------
            if (GrafikGas && GrafikWasser)
                if (Gaskurvendaten.Max(y => y.Y) > Wasserkurvendaten.Max(y => y.Y))
                    Grafik.Y2Axis.Scale.Max = Gaskurvendaten.Max(y => y.Y) * Skalenfaktor; else
                    Grafik.Y2Axis.Scale.Max = Wasserkurvendaten.Max(y => y.Y) * Skalenfaktor;

            if (GrafikGas && !GrafikWasser)
                Grafik.Y2Axis.Scale.Max = Gaskurvendaten.Max(y => y.Y) * Skalenfaktor;

            if (!GrafikGas && GrafikWasser)
                Grafik.Y2Axis.Scale.Max = Wasserkurvendaten.Max(y => y.Y) * Skalenfaktor;

    
            Grafik.YAxis.Scale.BaseTic = 0; //Grafik.YAxis.Scale.Max / 11;
            Grafik.Y2Axis.Scale.BaseTic = 0; // Grafik.Y2Axis.Scale.Max / 11; 

            Grafik.Y2Axis.Scale.MajorStep = Grafik.Y2Axis.Scale.Max / 10;
            Grafik.Y2Axis.Scale.MinorStep = Grafik.Y2Axis.Scale.Max / 5;

            Refresh();

            //---------------------------------------------
            // Start- und Enddatum an der Grafik ausgeben
            //---------------------------------------------
            // Müsste eigentlich zedgraph zurückliefern......
            DateTime Zeitpunkt = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
            Zeitpunkt = Zeitpunkt.AddSeconds (GrafikXStart);
            // LabelGrafikStart.Text = Zeitpunkt.ToShortDateString();
            dateTimePicker1.Value = Zeitpunkt;

            Zeitpunkt = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
            Zeitpunkt = Zeitpunkt.AddSeconds (GrafikXEnde);
            // LabelGrafikEnde.Text = Zeitpunkt.ToShortDateString();
            dateTimePicker2.Value = Zeitpunkt;

        }

        //---------------
        // Hauptformular
        //---------------
        public Form1()
        {
            InitializeComponent();
            
            // Kalenderformat für die Grafik festlegen
            dateTimePicker1.Format = DateTimePickerFormat.Custom;
            dateTimePicker1.CustomFormat = "dd.MMM.yyyy HH:mm";
            dateTimePicker2.Format = DateTimePickerFormat.Custom;
            dateTimePicker2.CustomFormat = "dd.MMM.yyyy HH:mm";
   
            // System konfigurieren
            LadeKonfiguration();

            // Standardwerte setzen
            labelAktuellesDatum.Text = System.DateTime.Now.ToShortDateString();

            // Vorhandene Daten einladen, Fehler beseitigen, Transientberechnungen und Randbedingungen ausgeben
            DateiLaden();
            Junkfilter();
            MomentDatenErrechnen();

            ZeigeDatenBasis();

            // Abgelesene Zählerdaten der Versorger einladen
            LadeKostenDaten();

            // Ablesezeiträume einladen
            LadeAbleseDaten();

            // Abschläge einladen
            LadeAbschlagsDaten();

            // Statistikwerte erzeugen
            ErrechneStatistikSeite();
  
            // Grafik in Betrieb nehmen
            InitGrafik();
            ErzeugeGrafik();

            // Zustand der Temperaturcheckbox festhalten
            TCheckbox = new int [8];
            TCheckboxAlt = new int [8];
                
            for (int i = 0; i < 8; i++)
            {
                TCheckbox[i] = TemperaturListBox1.CheckedIndices.IndexOf(0);
                TCheckboxAlt[i] = TemperaturListBox1.CheckedIndices.IndexOf(0);
            }
            


        }

   
        private void ButtonEinstellungKlick(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("config.txt");     // Konfigurationsdatei öffnen
            LadeKonfiguration();                                
            ErrechneStatistikSeite();
        }

        private void ButtonKostenBearbeitenKlick(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("kosten.txt");
            LadeKostenDaten();
        }

        private void ButtonAblesungenBearbeitenKlick(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("ablesungen.txt");
            LadeAbleseDaten();
        }

        private void ButtonAbschlaegeKlick(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("abschlag.txt");
            LadeAbschlagsDaten();
        }

        //--------------------------------------
        // Aktuelle Daten vom Webserver abholen
        //--------------------------------------
        private void DatenAbholenClick(object sender, EventArgs e)
        {
            int Counter = 0;
            uint utc;

            utc = lRohdaten.Last().UTC - 5 * 60; // Einen Datensatz rückwärts
            string WebServeradresse = "http://" + WebServerIP + "/daten.htm?start=" + Convert.ToString(utc);

            MessageBox.Show("Datenübertragung wird gestartet", "Daten abholen");

            // prepare the web page we will be asking for
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(WebServeradresse);
         try
            {
                // execute the request
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                if (response.StatusCode != HttpStatusCode.OK)
                    MessageBox.Show(response.StatusCode.ToString());
                // we will read data via the response stream
                Stream resStream = response.GetResponseStream();
                StreamReader reader = new StreamReader(resStream);

                //------------------------------------------------------------------------------------------
                // Reinkommende Daten zeilenweise laden, in Liste archivieren und an Rohdatendatei anhängen
                //------------------------------------------------------------------------------------------
                string Texteile = "";
                StreamWriter sw = File.AppendText("zaehlerdaten.txt");
                System.DateTime localDateTime = DateTime.UtcNow;
                DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
                TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln

                // Mein Freund der Fortschrittsbalken
                Abholbalken.Visible = true;
                Abholbalken.Minimum = 1;
                // Maximalwert ist die Anzahl der zu erwartenden Datensätze (Aktuelles Datum - Letztes vorhandenes Datum)
                int Maxwert = (int)(Convert.ToInt32(ts.TotalSeconds) - lRohdaten.Last().UTC) / (60 * 5);
                if (Maxwert < 2) Maxwert = 2;
                Abholbalken.Maximum = Maxwert;
                Abholbalken.Value = 1;
                Abholbalken.Step = 1;
               
                while ((!reader.EndOfStream) && (Texteile.IndexOf("EOM") < 0))
                {

                    // Counter++;
                    // Nächste Textzeile reinholen
                    Texteile = "";
                    while (Texteile.IndexOf(">") < 0) Texteile = Texteile + (char)reader.Read();
                    if (Texteile.IndexOf("\n") > 0)
                     Texteile = Texteile.Remove(0, 2); // Führendes \n\r entfernen
                
                    // System.Threading.Thread.Sleep(200); // 500ms Pause
 
                    
                    if ((Texteile != "") && (Texteile.IndexOf("#") < 0) && (Texteile != "EOM") && (Texteile.IndexOf("01.01.10") < 0) && (Counter > 0))
                    {
                        Texteile = Texteile.Remove(Texteile.IndexOf("<")); // Texteile = Texteile + Environment.NewLine;
                        lRohdaten.Add(new Zaehlerdaten(Texteile)); // Zeile in Liste zwischenspeichern
                        if (Texteile.IndexOf("EOM") < 0)  sw.WriteLine(Texteile);                     // Zeile in Datei speichern
                        Abholbalken.PerformStep();                  // Balken anzeigen
                        
                    }
                    Counter++;
                }
                Abholbalken.Visible = false;
                resStream.Close();                              // Stream wieder schliessen
                reader.Close();                                 // Stream wieder schliessen
                sw.Close();                                     // Datei wieder schliessen
                Junkfilter();                                   // Faule Elemente entsorgen
              
            
         }
           catch (Exception ex)
            {
              //Fehlermeldung
               MessageBox.Show("Der Webserver kann unter " + WebServeradresse + " nicht erreicht werden. Meldung : "+ex.ToString(), "Serverfehler");
            }

         // Wunderbare neue Daten ausgeben
         MessageBox.Show("Datenübertragung abgeschlossen (" + Convert.ToString(Counter) +" Datensätze)", "Daten abholen");
         ErrechneStatistikSeite();
         ZeigeDatenBasis();
    

        }

        private void label1_Click(object sender, EventArgs e)
        {

        }



        private void chart1_Click(object sender, EventArgs e)
        {

        }

        private void button1_Click(object sender, EventArgs e)
        {

        }

        private void button9_Click(object sender, EventArgs e)
        {

        }

        private void Auswertung_Click(object sender, EventArgs e)
        {

        }

        private void Form1_Load(object sender, EventArgs e)
        {

        }

        private void flowLayoutPanel1_Paint(object sender, PaintEventArgs e)
        {

        }

        private void label14_Click(object sender, EventArgs e)
        {

        }

        private void label74_Click(object sender, EventArgs e)
        {

        }

        private void zedGraphControl1_Load(object sender, EventArgs e)
        {

        }

        private void label68_Click(object sender, EventArgs e)
        {

        }

        private void label5_Click(object sender, EventArgs e)
        {

        }

        //---------------------------------
        // Grundlast-Kurve Ein/Ausschalten
        //---------------------------------
        private void ButtonGrundlastKlick(object sender, EventArgs e)
        {
            GrafikGrundlast++;
            if (GrafikGrundlast > 2) GrafikGrundlast = 0;
            ErzeugeGrafik();
        }
        
        //----------------------------
        // Stromkurve Ein/Ausschalten
        //----------------------------
        private void ButtonStromKlick(object sender, EventArgs e)
        {
            if (GrafikStrom == false) GrafikStrom = true; else GrafikStrom = false;
            ErzeugeGrafik();     
        }

        //--------------------------
        // Gaskurve Ein/Ausschalten
        //--------------------------
        private void ButtonGasKlick(object sender, EventArgs e)
        {
            if (GrafikGas == false) GrafikGas = true; else GrafikGas = false;
            ErzeugeGrafik();      
        }

        //-----------------------------
        // Wasserkurve Ein/Ausschalten
        //-----------------------------
        private void ButtonWasserKlick(object sender, EventArgs e)
        {
            if (GrafikWasser == false) GrafikWasser = true; else GrafikWasser = false;
            ErzeugeGrafik();
        }

        //----------------------------------------
        // Zeitraum für die Grafik wurde geändert
        //----------------------------------------
        private void ZeitraumChange(object sender, EventArgs e)
        {
            int test = ComboBoxZeitraum.SelectedIndex;

            switch (test)
            {
                // Heute
                case 0:
                    {   // Heute
                        // Lokale Zeitvariable erzeugen
                        System.DateTime localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());
                        DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
                        TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                        GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds);
                        GrafikXEnde = lRohdaten.Last().UTC;
                        break;
                    }
                // gestern
                case 1:
                    {   // gestern
                        // Lokale Zeitvariable erzeugen
                        System.DateTime localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());
                        DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
                        TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                        GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds) - 86400;
                        GrafikXEnde = GrafikXStart + 86400;
                        
                        break;
                    }
                // Diese Woche
                case 2:
                    {   // Diese Woche
                        System.DateTime localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());
                        int AnzahlWochentage = 1;
                        while ((int)localDateTime.DayOfWeek != 1)
                        {
                            localDateTime = localDateTime.AddDays(-1);    // Lokalzeiger auf den letzten Montag setzen
                            AnzahlWochentage++;                           // Anzahl vergangener Tage dieser Woche bestimmen  
                        }
                        DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
                        TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                        GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn der aktuellen Woche
                        GrafikXEnde = lRohdaten.Last().UTC;
                        break;
                    }
                // letzte Woche
                case 3:
                    {   // Letzte Woche
                        System.DateTime localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());
                        int AnzahlWochentage = 1;
                        while ((int)localDateTime.DayOfWeek != 1)
                        {
                            localDateTime = localDateTime.AddDays(-1);    // Lokalzeiger auf den letzten Montag setzen
                            AnzahlWochentage++;                           // Anzahl vergangener Tage dieser Woche bestimmen  
                        }
                        DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
                        TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                        GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn der aktuellen Woche
                        GrafikXEnde = lRohdaten.Last().UTC;

                        GrafikXEnde = GrafikXStart - 1;         // Ende der letzten Woche = Anfang dieser Woche-1
                        localDateTime = localDateTime.AddDays(-7);      // 7 Tage Rückwärts
                        ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                        GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn der letzten Woche
                        GrafikXEnde = GrafikXStart + 7 * 24 * 60 * 60 - 1;               // Eine Woche -1 drauf = Wochenende
                        break;
                    }
                // Dieser Monat
                case 4:
                    {   // Dieser Monat
                        System.DateTime localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());
                        int Testmonat = localDateTime.Month;
                        int AnzahlWochentage = 1;
                        // Tage bis zum Überschreiten der Monatsgrenze abziehen
                        while (Testmonat == localDateTime.Month)
                        {
                            localDateTime = localDateTime.AddDays(-1);    // Lokalzeiger auf den letzten Montag setzen
                            AnzahlWochentage++;                           // Anzahl vergangener Tage dieser Woche bestimmen  
                        }
                        // Verursachten Fehler wieder geradeziehen
                        AnzahlWochentage--; localDateTime = localDateTime.AddDays(1);
                        DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
                        TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                        GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
                        GrafikXEnde = lRohdaten.Last().UTC;
                        break;
                    }

                // Letzter Monat
                case 5:
                    {   // Letzter Monat
                        System.DateTime localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());
                        int Testmonat = localDateTime.Month;
                        int AnzahlWochentage = 1;
                        // Tage bis zum Überschreiten der Monatsgrenze abziehen
                        while (Testmonat == localDateTime.Month)
                        {
                            localDateTime = localDateTime.AddDays(-1);    // Lokalzeiger auf den letzten Montag setzen
                            AnzahlWochentage++;                           // Anzahl vergangener Tage dieser Woche bestimmen  
                        }
                        // Verursachten Fehler wieder geradeziehen
                        AnzahlWochentage--; localDateTime = localDateTime.AddDays(1);
                        DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
                        TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                        GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
                        GrafikXEnde = lRohdaten.Last().UTC;
                        localDateTime = localDateTime.AddMonths(-1);             // Einen Monat zurück
                        ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                        GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des vergangenen Monats
                        int tage = DateTime.DaysInMonth(localDateTime.Year, localDateTime.Month); // Wie viele Tage hat besagter Monat ?
                        GrafikXEnde = GrafikXStart + (uint)tage * 24 * 60 * 60 - 1;            // Ziegt jetzt auf das Ende des vergangen Monats
                        break;
                    }
                // Dieses Jahr
                case 6:
                    {   // Dieses Jahr
                        System.DateTime localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());  // Datum wieder geradeziehen         
                        int AnzahlTageimJahr = localDateTime.DayOfYear;                                     // Anzahl der vergangenen Tage in diesem Jahr
                        int Jahr = localDateTime.Year;
                        localDateTime = new DateTime(Jahr, 1, 1);                                           // Referenzdatum = Beginn des Jahres
                        DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
                        TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                        GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
                        GrafikXEnde = lRohdaten.Last().UTC;
                        break;
                    }
                // Letztes Jahr
                case 7:
                    {   // Letztes Jahr
                        System.DateTime localDateTime = System.DateTime.Parse(System.DateTime.UtcNow.ToShortDateString());  // Datum wieder geradeziehen         
                        int Jahr = localDateTime.Year;
                        localDateTime = new DateTime(Jahr - 1, 1, 1);                                        // Referenzdatum = Beginn des letzten Jahres
                        int AnzahlTageimJahr = GetDaysInAYear(Jahr - 1);
                        DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
                        TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
                        GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds);       // Zeigt jetzt auf den Beginn des aktuellen Monats
                        GrafikXEnde = GrafikXStart + (uint)AnzahlTageimJahr * 24 * 60 * 60;
                        break;
                    }
       
       
            
            } // Von Switch

            //-----------------------------------------
            // Neuscalierung der X-Achse durchführen
            //-----------------------------------------
            if (GrafikXStart > GrafikXEnde) GrafikXEnde = GrafikXStart;

            // Umrechnung der UCT-Werte für Start- und Ende in das ZEDGraph-eigene Datumsformat
            // Start
            DateTime Zeitpunkt = new DateTime(1970, 1, 1);                      
            Zeitpunkt = Zeitpunkt.AddSeconds(GrafikXStart);
            Grafik.XAxis.Scale.Min = (double)new XDate(Zeitpunkt.Year, Zeitpunkt.Month, Zeitpunkt.Day, Zeitpunkt.Hour, Zeitpunkt.Minute, Zeitpunkt.Second);
            // Ende
            Zeitpunkt = new DateTime(1970, 1, 1);                      
            Zeitpunkt = Zeitpunkt.AddSeconds(GrafikXEnde);
            Grafik.XAxis.Scale.Max = (double)new XDate(Zeitpunkt.Year, Zeitpunkt.Month, Zeitpunkt.Day, Zeitpunkt.Hour, Zeitpunkt.Minute, Zeitpunkt.Second);
            
            // Grafik neu aufbauen
            ErzeugeGrafik();

        }

        //--------------------------
        // Grafik wurde angeklickt
        //--------------------------
        private void Grafikklick(object sender, MouseEventArgs e)
        {
            double checker = Grafik.XAxis.Scale.Min;
        }
        
        //-----------------------------------
        // In der Grafik wurde herumgezoomt
        //-----------------------------------
        private void zedGraphControl1_ZoomEvent(ZedGraphControl sender, ZoomState oldState, ZoomState newState)
        {
            System.DateTime localDateTime = XDate.XLDateToDateTime(Grafik.XAxis.Scale.Min);
            DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
            TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds);

            localDateTime = XDate.XLDateToDateTime(Grafik.XAxis.Scale.Max);
            RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
            ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            GrafikXEnde = (uint)Convert.ToInt32(ts.TotalSeconds);       
            // Grafik neu aufbauen
            ErzeugeGrafik();
        }

        //-----------------------------------
        // Schaltet Temperaturkurve  ein/aus
        //-----------------------------------
        void TKurve (bool AnAus, int index)
        {
            if (AnAus == true)
            {
                Grafik.CurveList[index + 5].IsVisible = true;       // Grafik ist sichtbar
                // yAxis3.IsVisible = true;                    // Y-Achse ist sichtbar
                Grafik.CurveList[index + 5].YAxisIndex = 1;
            }
            else
            {
                Grafik.CurveList[index + 5].IsVisible = false;              // Grafik ist unsichtbar    
                // yAxis3.IsVisible = false;                           // Y-Achse ist sichtbar
            }
        }

  
        //-----------------------------------------------------
        // Aufgerufen bei Umschaltung der TemperaturenCheckbox
        //-----------------------------------------------------
        private void TemperaturenWechel(object sender, EventArgs e)
        {
            // Aktuellen Zustand reinholen
            bool Achsezeigen = false;
            for (int i = 0; i < 8; i++)
            {
                TCheckbox[i] = TemperaturListBox1.CheckedIndices.IndexOf(i);
                if (TCheckbox[i] != TCheckboxAlt[i])
                {
                    TCheckboxAlt[i] = TCheckbox[i];                                     // Zustand sichern
                    if (TCheckbox[i] != -1) { Achsezeigen = true; TKurve(true, i); }  else TKurve(false, i);     // Kurve zeichnen oder löschen je nach "Haken"
                }
            }
            // Y-Achse ist sichtbar oder nicht 
            for (int i = 0; i < 8; i++)
                if (TCheckbox[i] != -1) Achsezeigen = true;
            if (Achsezeigen == true) yAxis3.IsVisible = true; else yAxis3.IsVisible = false;

            // Achse skalieren falls notwendig
            double [] min, max;
            min = new double [8];
            max = new double [8];
            double gesmin;
            double gesmax;

            // Defaultwerte setzen
            gesmin = 10;
            gesmax = 20;

            min [0] = KurveAussen.Min(y => y.Y);
            min [1] = KurveHeizungVorlauf.Min(y => y.Y);
            min [2] = KurveHeizungRuecklauf.Min(y => y.Y);
            min [3] = KurveBoilerEinlass.Min(y => y.Y);
            min [4] = KurveBoilerAuslass.Min(y => y.Y);
            min [5] = KurveAbgasHeizung.Min(y => y.Y);
            min [6] = KurveAbgasBoiler.Min(y => y.Y);
            min [7] = KurveHeizungsraum.Min(y => y.Y);

            max [0] = KurveAussen.Max(y => y.Y);
            max [1] = KurveHeizungVorlauf.Max(y => y.Y);
            max [2] = KurveHeizungRuecklauf.Max(y => y.Y);
            max [3] = KurveBoilerEinlass.Max(y => y.Y);
            max [4] = KurveBoilerAuslass.Max(y => y.Y);
            max [5] = KurveAbgasHeizung.Max(y => y.Y);
            max [6] = KurveAbgasBoiler.Max(y => y.Y);
            max [7] = KurveHeizungsraum.Max(y => y.Y);

            // Listen durchgehen
            for (int i = 0; i < 8; i++)
                if (TCheckbox[i] != -1)
                {
                    if (min[i] < gesmin) gesmin = min[i];
                    if (max[i] > gesmax) gesmax = max[i]; 
                }
            yAxis3.Scale.Min = gesmin;
            yAxis3.Scale.Max = gesmax;

            yAxis3.Scale.MajorStep = (gesmax - gesmin) / 10;
            // YAxis.Scale.BaseTic

            // Grafik neu aufbauen scheisse
            ErzeugeGrafik();
   
                 /*
                0 Aussen
                1 HeizungVorlauf
                2 HeizungRücklauf
                3 BoilerEinlass
                4 BoilerAuslass
                5 AbgasHeizung
                6 AbgasBoiler
                7 Heizungsraum
                */
        
        }

        //----------------------------
        // Startzeitkalender geändert
        //----------------------------
        private void KalenderStart(object sender, EventArgs e)
        {
            // Neues Startdatum für Grafik errechnen
            System.DateTime localDateTime = dateTimePicker1.Value;
            DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
            TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            GrafikXStart = (uint)Convert.ToInt32(ts.TotalSeconds);

            //-----------------------------------------
            // Neuscalierung der X-Achse durchführen
            //-----------------------------------------
            // if (GrafikXStart > GrafikXEnde) GrafikXEnde = GrafikXStart;
            // scheisse
            // Umrechnung der UCT-Werte für Start- und Ende in das ZEDGraph-eigene Datumsformat
            // Start
            DateTime Zeitpunkt = new DateTime(1970, 1, 1);
            Zeitpunkt = Zeitpunkt.AddSeconds(GrafikXStart);
            Grafik.XAxis.Scale.Min = (double)new XDate(Zeitpunkt.Year, Zeitpunkt.Month, Zeitpunkt.Day, Zeitpunkt.Hour, Zeitpunkt.Minute, Zeitpunkt.Second);
            // Ende
            Zeitpunkt = new DateTime(1970, 1, 1);
            Zeitpunkt = Zeitpunkt.AddSeconds(GrafikXEnde);
            Grafik.XAxis.Scale.Max = (double)new XDate(Zeitpunkt.Year, Zeitpunkt.Month, Zeitpunkt.Day, Zeitpunkt.Hour, Zeitpunkt.Minute, Zeitpunkt.Second);

            // Grafik neu aufbauen
            ErzeugeGrafik();

        }

        //----------------------------
        // Endzeitkalender geändert
        //----------------------------
        private void KalenderEnde(object sender, EventArgs e)
        {
            // Neues Startdatum für Grafik errechnen
            System.DateTime localDateTime = dateTimePicker2.Value;
            DateTime RefDate = new DateTime(1970, 1, 1);                      //Referenzdatum (festgelegt)
            TimeSpan ts = new TimeSpan(localDateTime.Ticks - RefDate.Ticks);  // das Delta ermitteln
            GrafikXEnde = (uint)Convert.ToInt32(ts.TotalSeconds);

            //-----------------------------------------
            // Neuscalierung der X-Achse durchführen
            //-----------------------------------------
            // if (GrafikXStart > GrafikXEnde) GrafikXEnde = GrafikXStart;

            // Umrechnung der UCT-Werte für Start- und Ende in das ZEDGraph-eigene Datumsformat
            // Start
            DateTime Zeitpunkt = new DateTime(1970, 1, 1);
            Zeitpunkt = Zeitpunkt.AddSeconds(GrafikXStart);
            Grafik.XAxis.Scale.Min = (double)new XDate(Zeitpunkt.Year, Zeitpunkt.Month, Zeitpunkt.Day, Zeitpunkt.Hour, Zeitpunkt.Minute, Zeitpunkt.Second);
            // Ende
            Zeitpunkt = new DateTime(1970, 1, 1);
            Zeitpunkt = Zeitpunkt.AddSeconds(GrafikXEnde);
            Grafik.XAxis.Scale.Max = (double)new XDate(Zeitpunkt.Year, Zeitpunkt.Month, Zeitpunkt.Day, Zeitpunkt.Hour, Zeitpunkt.Minute, Zeitpunkt.Second);

            // Grafik neu aufbauen
            ErzeugeGrafik();
        }

        //-------------------------------------------------------------------------
        // Statistik für die eingeblendeten Kurven anzeigen im aktuellen Zeitraum
        //-------------------------------------------------------------------------
        private void ButtonStatistikKlick(object sender, EventArgs e)
        {
           Form2 formular = new Form2();
           uint Zeitraum;
           uint tage, stunden;

            // Start Grafik
            System.DateTime dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
           dateTime = dateTime.AddSeconds(GrafikXStart);
           string temptext = dateTime.ToShortDateString();
           temptext += " "+dateTime.ToShortTimeString() + " Uhr";
           formular.StatistikVon = temptext;

           // Ende Grafik
           dateTime = new System.DateTime(1970, 1, 1, 0, 0, 0, 0);
           dateTime = dateTime.AddSeconds(GrafikXEnde);
           temptext = dateTime.ToShortDateString();
           temptext += " " + dateTime.ToShortTimeString() + " Uhr";
           formular.StatistikBis = temptext;

            // Zeitraum
           Zeitraum = GrafikXEnde - GrafikXStart;      // Zeitraum in Sekunden
           tage = Zeitraum / (24 * 60 * 60);
           stunden = (Zeitraum - (tage*24*60*60)) / (60 * 60);
           formular.StatistikZeitraum = Convert.ToString(tage) + " Tage " + 
                                        Convert.ToString(stunden) + " Stunden"; 

           // Gesamte Liste mit Rohdaten durchkämmen um den Zeitraum zu finden
           Strom = SummationStrom(GrafikXStart, GrafikXEnde);
           Gas = SummationGas(GrafikXStart, GrafikXEnde);
           Wasser = SummationWasser(GrafikXStart, GrafikXEnde);

           // Aktuelle Tagesverbräuche bestimmen und mit Zählerkonstanten verrechnen
           RealWertbestimmung();

           // Daten ausgeben
           formular.StatistikStrom = Strom.ToString("F1") + " kWh";
           formular.StatistikGas = Gas.ToString("F1") + " qm";
           formular.StatistikWasser = Wasser.ToString("F0") + " Liter";
           
           // Umrechnung in entstandene Kosten 

           float Stromkosten = Strom * lKostendaten.Last().StromArbeitspreis;
           float Grundkosten = (lKostendaten.Last().StromGrundpreis / (365 * 24 * 60 * 60));
           Stromkosten += Grundkosten * Zeitraum;
           temptext = Stromkosten.ToString ("#,##0.00") + " €";
           formular.StatistikStromGeld = temptext;

           float Gaskosten = Gas * lKostendaten.Last().Gararbeitspreis * lKostendaten.Last().GasUmrechnungsfaktor;
           Gaskosten += (lKostendaten.Last().GasGrundpreis / (365 * 24 * 60 * 60)) * Zeitraum;
           temptext = Gaskosten.ToString ("#,##0.00") + " €";
           formular.StatistikGasGeld= temptext;

           float Wasserkosten = (Wasser / 1000) * lKostendaten.Last().WasserArbeitspreis;
           Wasserkosten += (lKostendaten.Last().WasserGrundpreis / (365 * 24 * 60 * 60)) * Zeitraum;
           temptext = Wasserkosten.ToString ("#,##0.00") + " €";
           formular.StatistikWasserGeld= temptext;

           // Und was kostet der Spaß ?
           float AllesZusammen = Stromkosten + Gaskosten + Wasserkosten;
           temptext = AllesZusammen.ToString("#,##0.00") + " €";
           formular.StatistikGeldSumme = temptext;
            
           // Formular anzeigen
           formular.Show();


        }

        private void ButtonExportClick(object sender, EventArgs e)
        {
            MessageBox.Show("Ist noch nicht fertig!", "Datenexport");
        }

        private void LabelzaehlerstandGas_Click(object sender, EventArgs e)
        {

        }

        private void LabelzaehlerstandStrm_Click(object sender, EventArgs e)
        {

        }

    }
}

